/***************************************************************************
 *            book-fun.c
 *
 *  Sun Jul 10 19:14:16 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "book-fun.h"
#include "support.h"
#include "db-fun.h"
#include "main-callbacks.h"

/*
 *	Funkcja odczytuje dane ksiazki o wskazanym ID
 */

void	select_book_data	(gint book_id)	{
	sqlite3_stmt	*db_row;
	gchar			sql_query[64];
	gint			status;

	g_sprintf (sql_query, "SELECT * FROM books WHERE id = %d;", book_id);
	sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);
	
	status = sqlite3_step (db_row);
	g_printf(">>> Selecting book data with ID = %d, SQLite result %d\n", book_id, status);
	
	book.id = sqlite3_column_int (db_row, 0);
	g_sprintf (book.title, "%s", sqlite3_column_text (db_row, 1));
	book.author = sqlite3_column_int (db_row, 2);
	book.lended = sqlite3_column_int (db_row, 3);
	g_sprintf (book.description, "%s", sqlite3_column_text (db_row, 4));
	
	sqlite3_finalize (db_row);
}


/*
 *	Funkcja zapisuje/aktualizuje dane o ksiazce
 *	Wartosci 'save_mode' zadeklarowanej w 'raw_book-fun.h':
 *		1 - dodawanie ksiazek
 *		2 - aktualizacja pol 'title', 'author', 'description'
 */

void	save_book (void)	{
	gchar			sql_query[1280];
	gint			status;
	sqlite3_stmt	*db_row;
		
	g_printf (_(">>> Updating data base ...\n"));
	
	switch (save_mode)	{
		case 1:
			g_sprintf (sql_query, "INSERT INTO books (id, title, author, lended, description) VALUES (NULL, \'%s\', \'%d\', \'0\', \'%s\');", book.title, book.author, book.description);
			sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);
			status = sqlite3_step (db_row);
			g_printf("\t<<< Inserting new data into data base, SQLIte result: %d\n", status);
			set_statusbar_text (_("New book inserted!"));
			break;
		case 2:
			g_sprintf (sql_query, "UPDATE books SET title=\'%s\', author=\'%d\', description=\'%s\' WHERE id=%d;", book.title, book.author, book.description, book.id);
			sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);
			status = sqlite3_step (db_row);
			g_printf("\t<<< Updating data in data base, SQLIte result: %d\n", status);
			set_statusbar_text (_("Book ingormations updated!"));
			break;
		case 3:
			g_sprintf (sql_query, "UPDATE books SET lended=\'%d\' WHERE id=%d;", book.lended, book.id);
			sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);
			status = sqlite3_step (db_row);
			g_printf("\t<<< Updating data in data base, SQLIte result: %d\n", status);
			set_statusbar_text (_("Book ingormations updated!"));
			break;
		default:
			set_statusbar_text (_("Unknown problem while adding/updating book!"));
			g_printf ("\t<<<Unknown problem!\n");
	}
	
	sqlite3_finalize (db_row);
	
	save_mode = 0;
}


/*
 *	Funkcja usuwa ksiazke o wskazanym 'book_id' z tabeli 'books' w bazie danyh.
 */

void	remove_book					(gint	book_id)	{
	gchar			sql_query[64];
	gint			status;
	sqlite3_stmt	*db_row;

	g_sprintf (sql_query, "DELETE FROM books WHERE id=%d;", book_id);
	sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf(">>> Deleting selected book data from database, SQLite result: %d\n", status);
	set_statusbar_text (_("Book removed from data base."));
	sqlite3_finalize (db_row);
}

/*
 *	Funkcja wczytuje tytuly filmow do listy 'main_view' z bazy danych
 */

void	load_books_titles	(void)	{
	GtkTreeView			*treeview;
	GtkTreeModel		*store;
	GtkTreeIter 		iter;	
	gint				status = 0;
	gint				rows;
	gchar				*lended;
	sqlite3_stmt		*db_row;
		
	treeview = (gpointer) lookup_widget (main_window, "main_tview");
	store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
	
	g_printf(">>> Loading books titles, counting books...\n");
	rows = count_returned_rows ("SELECT id FROM books;");
	g_printf ("<<< Books in data base: %d\n", rows);
	
	sqlite3_prepare(db_handler, "SELECT id, title, lended FROM books;", -1, &db_row, 0);	
	
	status = sqlite3_step (db_row);
	g_printf ("\t<<< Got first book, SQLite result: %d\n", status);
	if (rows > 0)	{
		while (status == 100)		{
			if (sqlite3_column_int (db_row, 2) == 0)	{
				lended = "";
			}
			else	{
				lended = _("Yes");
			}
			
			gtk_list_store_append (GTK_LIST_STORE (store), &iter);
			gtk_list_store_set (GTK_LIST_STORE (store), &iter,
								0, sqlite3_column_int (db_row, 0), 
								1, sqlite3_column_text (db_row, 1),
								2, lended,
								-1);
			status = sqlite3_step (db_row);
		}
		g_printf ("\t<<< Got last book, SQLite result: %d\n", status);
	}
	
	sqlite3_finalize (db_row);
}


/*
 *	Funkcja wczytuje dane wskazanej ksiazki do pol w 'main_win' i tworzy modele
 *	dla wszystkich widgetow 'GtkComboBox' wybierajha przy tym opce zapisana w 
 *	bazie danych
 */

void	display_book_data	(void)	{
	GtkWidget		*tmp;
	GtkTextBuffer 	*text_buffer;
	gchar			sql_query	[64];
	sqlite3_stmt	*db_row;
	
	tmp = (gpointer) lookup_widget (main_window, "book_title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), book.title);

	tmp = (gpointer) lookup_widget (main_window, "author_book_lb");
	g_sprintf (sql_query, "SELECT author FROM authors WHERE id=\'%d\';", book.author);
	sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);	
	sqlite3_step (db_row);
	gtk_label_set_text (GTK_LABEL (tmp), sqlite3_column_text (db_row, 0));
	sqlite3_finalize (db_row);
	
	if (book.lended == 0)	{
		tmp = (gpointer) lookup_widget (main_window, "lend_book_togbut");
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), FALSE);
	}
	else	{
		tmp = (gpointer) lookup_widget (main_window, "lend_book_togbut");
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), TRUE);
	}
	
	tmp = (gpointer) lookup_widget (main_window, "desc_book_text_view");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, book.description, strlen (book.description));
}

/*
 *	Funkcja czysci pola w oknie 'main_win'
 */

void	clear_book_display (void)	{
	GtkWidget		*tmp;
	GtkTextBuffer 	*text_buffer;
	
	tmp = (gpointer) lookup_widget (main_window, "book_title_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");

	tmp = (gpointer) lookup_widget (main_window, "author_book_lb");
	gtk_label_set_text (GTK_LABEL (tmp), "");
	
	tmp = (gpointer) lookup_widget (main_window, "desc_book_text_view");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (tmp));
	gtk_text_buffer_set_text (text_buffer, "", 0);
	
	set_widget_sensinitive (main_window, "rem_book_but", FALSE);
	set_widget_sensinitive (main_window, "edit_book_but", FALSE);
	set_widget_sensinitive (main_window, "lend_book_togbut", FALSE);
}

/*
 *	Funkcja laduje modele z tabel 'authors' do odpowiednich widgetow 'GtkEntry'
 */

void	create_add_dlg_data			(GtkWidget	*widget)	{
	GtkListStore		*list;
	GtkTreeIter			iter;
	GtkEntryCompletion	*completion;
	GtkWidget			*tmp;
	gint				status;
	gint				rows;
	sqlite3_stmt		*db_row;
	
	g_printf (">>> Loading base data into 'add_dlg'...\n");
	
	/*=====   GtkEntry 'author_en'   =====*/
	tmp = (gpointer) lookup_widget (widget, "author_en");
	
	rows = count_returned_rows ("SELECT id FROM authors;");
	if (rows > 0)	{
		sqlite3_prepare(db_handler, "SELECT author FROM authors;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading first row table \'authors\', SQLite result: %d\n", status);
	
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
			g_printf ("\t<<< Loading last row table \'authors\', SQLite result: %d\n", status);
		}
		
		sqlite3_finalize (db_row);
		
		gtk_combo_box_set_model (GTK_COMBO_BOX (tmp), GTK_TREE_MODEL (list));
		
		tmp = (gpointer) GTK_ENTRY (GTK_BIN (tmp)->child);
		
		completion = gtk_entry_completion_new ();
		gtk_entry_completion_set_inline_completion (completion, TRUE);
		gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (list));
		gtk_entry_completion_set_text_column (completion, 0);
		gtk_entry_set_completion (GTK_ENTRY (tmp), completion);
	}
}

/*
 *	Funkcja wczytuje dane wskazanej ksiazki do okna edycji. 
 */

void	load_data_into_add_book_dlg (GtkWidget *widget)	{
	GtkWidget			*tmp;
	GtkTextBuffer		*text_buffer;
	sqlite3_stmt		*db_row;
	gchar				sql_query[64];
	
	tmp = (gpointer) lookup_widget (widget, "title_en");
	gtk_entry_set_text (GTK_ENTRY (tmp), book.title);

	tmp = (gpointer) lookup_widget (widget, "author_en");
	g_sprintf (sql_query, "SELECT author FROM authors WHERE id=\'%d\';", book.author);
	sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);	
	sqlite3_step (db_row);
	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (tmp)->child), sqlite3_column_text (db_row, 0));
	sqlite3_finalize (db_row);
	
	widget = (gpointer) lookup_widget (GTK_WIDGET (tmp), "desc_en");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
	gtk_text_buffer_set_text (text_buffer, book.description, strlen (book.description));
}

/*
 *	Funkcja wyciaga dane o ksiazkach z okna 'add_book_dlg' i uaktualnia
 *	strukture 'book'
 */

void	update_book_data_from_dlg	(GtkWidget	*widget)	{
	GtkWidget		*tmp;
	GtkTextBuffer 	*text_buffer;
	GtkTextIter		text_start;	
	GtkTextIter		text_end;
	sqlite3_stmt	*db_row;
	gchar			sql_query[128];
	
	tmp = (gpointer) lookup_widget (widget, "title_en");
	g_sprintf (book.title, "%s", gtk_entry_get_text (GTK_ENTRY (tmp)));

	tmp = (gpointer) lookup_widget (widget, "author_en");
	g_sprintf (sql_query, "SELECT id FROM authors WHERE author=\'%s\';", 
						  gtk_entry_get_text (GTK_ENTRY (GTK_BIN (tmp)->child)));
	sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);	
	sqlite3_step (db_row);
	book.author =  sqlite3_column_int (db_row, 0);
	sqlite3_finalize (db_row);
	
	widget = (gpointer) lookup_widget (GTK_WIDGET (tmp), "desc_en");
	text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
	gtk_text_buffer_get_bounds (text_buffer, &text_start, &text_end);
	g_sprintf (book.description, "%s", gtk_text_buffer_get_text (text_buffer, &text_start, &text_end, TRUE));
}
