/***************************************************************************
 *            db-fun.c
 *
 *  Sun Jul 10 14:05:51 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "db-fun.h"
#include "support.h"
#include "utility-fun.h"

/*
 *	Funkcja wywolywana jest podczas pierwszego uruchomienia programu; tworzy
 *	niezbedne pliki i tabele w bazie danych oraz wstawia podstawowe dane (kilka
 *	gatunkow ksiazek).
 */
 
void	first_run							(gchar db_path[1024])	{
	sqlite3_stmt	*db_row;
	gint			status;
	gchar			command	[1024];
	
	g_printf ("This is first run of GBiblioteka, program must create base files, please wait...\n");
	
	g_sprintf (command, "%s/.gbiblioteka", g_get_home_dir ());
	status = g_mkdir (command, 0711);
	
	status = sqlite3_open (db_path, &db_handler);
	g_printf ("<<< Opening data base, SQLite result: %d\n", status);

	status = sqlite3_prepare(db_handler, "CREATE TABLE books (id INTEGER PRIMARY KEY AUTOINCREMENT, title TEXT, author INTEGER, lended INTEGER, description TEXT);", -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'books\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status = sqlite3_prepare(db_handler, "CREATE TABLE authors (id INTEGER PRIMARY KEY AUTOINCREMENT, author TEXT);", -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'authors\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	g_printf (">>> Inserting base data into database...\n");
	/* !!! Please translate only 'Somebody' word !!! */
	status = sqlite3_prepare(db_handler, _("INSERT INTO authors (id, author) VALUES (NULL, \'Somebody\');"), -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("\t<<< Table \'authors\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
}

/*
 *	Funkcja otwiera polaczenie z baza danych
 */

void	open_db		(void)	{
	gchar	db_path	[1024];
	gint	status;
	
	g_sprintf (db_path, "%s/.gbiblioteka/books.db", g_get_home_dir ());
	
	if (!g_file_test (db_path, G_FILE_TEST_EXISTS))	{	
		first_run (db_path);
	}
	else	{
		status = sqlite3_open (db_path, &db_handler);
		g_printf ("Opening data base, SQLite result: %d\n", status);
	}
}

/*
 *	Funkcja zamyka polaczenie z baza danych
 */

void	close_db (void)	{
	gint	status;
	
	status = sqlite3_close (db_handler);
	
	g_printf ("Closing data base, SQLite result: %d\n", status);
}

/*
 *	Funkcja zwraca ilosc wqierszy zwroconych przez wywolaniew SQL
 */

gint	count_returned_rows	(gchar	sql_query [1024])	{
	gint			rows;
	gint			status;
	sqlite3_stmt	*db_row;
	
	rows = 0;
	sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);	
	status = sqlite3_step (db_row);
	while (status == 100)	{
		++rows;
		status = sqlite3_step (db_row);
	}
	sqlite3_finalize (db_row);
	
	return rows;
}

/*
 *	Funkcja tworzy kopie pliku bazy danych i umieszcza ja w katalogu ~/.gbiblioteka
 *	doklejajac na koncu date utworzenia.
 */

void	export_data_from_db					(GtkWidget	*widget)	{
	gchar			string[1024];

	sprintf (string, "%s/%d-%d-%d.gbib", g_get_home_dir (), day, (++month), year);
	
	if (g_file_test (string, G_FILE_TEST_EXISTS))	{ /* jezeli plik istnieje... */
		show_message_dlg ("Export file is already exist and it's seems to be up to date!", 1);
	}
	else	{
		gdouble			step;
		gint			rows;
		gint			i;
		gint			status;
		GtkWidget		*progress;
		FILE			*file;
		sqlite3_stmt	*db_row;
		
		progress = (gpointer) lookup_widget (widget, "export_pbar");
	
		rows = count_returned_rows ("SELECT id FROM books;");

		if (rows == 0)	{
			g_printf ("\t<<< You heve no books in database.\n");
			set_statusbar_text (_("Export books aborted, you heve no books in database."));
			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress), 1.0);
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progress), _("Aborted, you have no books in database."));
		}
		else	{
			set_widget_sensinitive (widget, "export_close_but", FALSE);
			
			step = 1.0 / rows;
		
			file = fopen (string, "w");
			if(file == NULL)		{
				file = fopen (string, "w");
			}
			if(file == NULL)	{
				g_printf("\t<<<Can't save file with data!!!\n");
				set_statusbar_text (_("Can't save import file!"));
			}
			else	{		
				g_fprintf (file, "%d %d-%d-%d\n", rows, day, month, year);
				i = 1;
			
				sqlite3_prepare(db_handler, "SELECT title, author, description FROM books;", -1, &db_row, 0);
				status = sqlite3_step (db_row);
				if (status == 101)	{
					g_fprintf (file, "%s|||%s|||%s|END|\n", sqlite3_column_text (db_row, 0),
												   			sqlite3_column_text (db_row, 1),
											 	   			sqlite3_column_text (db_row, 2));
					gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress), 1.0);
				}
				else	{
					while (status == 100)	{
						g_fprintf (file, "%s|||%s|||%s|END|\n", sqlite3_column_text (db_row, 0),
												   				sqlite3_column_text (db_row, 1),
											 	   				sqlite3_column_text (db_row, 2));
						status = sqlite3_step (db_row);
						gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress),
										((gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (progress)) + step)));
						g_sprintf (string, _("%d / %d items saved"), i, rows);
						gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progress), string);
						++i;
					}
				}
			}
			g_sprintf (string, _("Succesfull!\nFile \'%d-%d-%d.gbib\' was created in your home directory."), day, month, year);
			show_message_dlg (string, 0);
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progress), _("Database converted and saved succesfull!"));
			g_printf ("\tDatabase saved!\n");
			sqlite3_finalize (db_row);
			fclose (file);
			set_widget_sensinitive (widget, "export_close_but", TRUE);			
		}
	}
}


void	backup_database				(GtkWidget	*widget)	{
	gchar		prefix		[8];
	gchar		sufix		[1024];
	gchar		file_path	[1024];
	GtkWidget	*file_chooser;
	
	file_chooser = (gpointer) lookup_widget (GTK_WIDGET (widget), "file_fchobut");
	g_sprintf (file_path, "%s", gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser)));
	
	if	(file_path [strlen (file_path) - 4] == 'g' &&
		 file_path [strlen (file_path) - 3] == 'b' &&
		 file_path [strlen (file_path) - 2] == 'i' &&
		 file_path [strlen (file_path) - 1] == 'b')	
	{
		gchar			title 		[128];
		gchar			author		[64];
		gchar			description	[1024];
		gchar			sql_query	[2048];
		gchar			string		[32];
		
		gint			i;
		gint			status;
		guint 			rows;
		guint			year;
		guint			month;
		guint			day;
		gdouble			step;
		
		FILE 			*file;
		sqlite3_stmt	*db_row;
		GtkWidget		*progress;
		
		progress = (gpointer) lookup_widget (widget, "backup_pbar");
		
		file = fopen (file_path, "r");
		if(file == NULL)		{
			file = fopen (file_path, "r");
		}
		if(file == NULL)	{
			printf("<<< Can't open file with data!!!\n");
			set_statusbar_text (_("Can't restore database!"));
		}
		else	{
			g_sprintf (file_path, "%s/.gbiblioteka/books.db", g_get_home_dir ());
			
			close_db ();
			if (g_remove (file_path) == -1)	{
				set_statusbar_text (_("Damn, something was wrong, database can be broken, sory! :-|"));
				g_printf ("Damn, something was wrong, database can be broken, sory! :-|\n");
			}
			else	{
				set_widget_sensinitive (widget, "backup_close_but", FALSE);
				open_db ();
				
				fscanf (file, "%d %d-%d-%d\n", &rows, &day, &month, &year);
				
				step = 1.0 / rows;
				
				i = 0;
				
				while (i < rows)	{		
					fscanf (file, "%[^|||]|||%[^|||]|||%[^|END|]|END|\n", title, author, description);
					g_sprintf (sql_query, "INSERT INTO books (id, title, author, description) VALUES (NULL, \'%s\', \'%s\', \'%s\');", title, author, description);
		
					sqlite3_prepare(db_handler, sql_query, -1, &db_row, 0);
					status = sqlite3_step (db_row);
					g_printf (_("SQLite result: %d\n"), status);
					sqlite3_finalize (db_row);
					
					gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress),
								((gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (progress)) + step)));			
					++i;
			
					g_sprintf (string, _("%d / %d items restored"), i, rows);
					gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progress), string);
				}
			}
		}
		fclose (file);
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progress), _("Database restored succesful!"));
		
		set_widget_sensinitive (progress, "backup_close_but", TRUE);
		
		show_message_dlg (_("After restoring database you should\nrestart GBiblioteka, you can do it now."), 1);
	}
	else	{
		show_message_dlg (_("File you choosed isn't Gbiblioteka backup file"), 1);
	}
}
