/***************************************************************************
 *            settings-fun.c
 *
 *  Sun Jul 10 13:22:36 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <gtk/gtk.h>
 
#include "main-callbacks.h"
#include "settings-fun.h"
#include "support.h"
#include "interface.h"
#include "gui-fun.h"
#include "other-callbacks.h"
#include "main-callbacks.h"
#include "utility-fun.h"

/*
 *	Funkcja odczytuje ustawienia z pliku 'gbiblioteka.conf'
 */

gboolean	load_settings_file	(void)	{
	gchar settings_file_path[1024];
	g_sprintf (settings_file_path, "%s/.gbiblioteka/%s", g_get_home_dir(), "gbiblioteka.conf");
	
	if( g_file_test (settings_file_path, G_FILE_TEST_EXISTS))		{
		GError		*error_handler;
		GKeyFile	*settings_file;
		
		error_handler = NULL;
		
		settings_file = g_key_file_new ();
		
		g_key_file_load_from_file (settings_file,
								   settings_file_path,
								   G_KEY_FILE_NONE,
								   &error_handler);	

/*================================================*
 *			Grupa 'main window position'		  *
 *================================================*/
		
		settings.remember = g_key_file_get_boolean (settings_file,
								"main window position",
								"remember",
								&error_handler);
		
		settings.x = g_key_file_get_integer (settings_file,
								"main window position",
								"x",
								&error_handler);
								
		settings.y = g_key_file_get_integer (settings_file,
								"main window position",
								"y",
								&error_handler);

/*================================================*
 *			Grupa 'security copy'				  *
 *================================================*/
 
 		settings.auto_copy = g_key_file_get_boolean (settings_file,
								"security copy",
								"auto_copy",
								&error_handler);
		
		settings.frequency = g_key_file_get_integer (settings_file,
								"security copy",
								"frequency",
								&error_handler);
								
		settings.next_copy_month = g_key_file_get_integer (settings_file,
								"security copy",
								"date",
								&error_handler);
		
		g_key_file_free (settings_file);
		g_clear_error (&error_handler);
		return TRUE;
	}
	else	{
		g_printf (_("(ERROR) Can't open config file!\n"));
		return FALSE;
	}		
}

/*
 *	Funkcja zapisuje ustawienia do pliku 'gbiblioteka.conf'
 */

void		save_settings	(void)	{
	gchar settings_file_path[1024];
	gchar string [2048];
	
	g_sprintf (settings_file_path, "%s/.gbiblioteka/%s", g_get_home_dir(), "gbiblioteka.conf");

	FILE *settings_file;
	
	settings_file = fopen (settings_file_path, "w");
	
	if(settings_file == NULL)		{
		g_mkdir ("~/.gbiblioteka", 740);
		
		settings_file = fopen (settings_file_path, "w");
	}
	if(settings_file == NULL)	{
		set_statusbar_text (_("Error while saving settings!"));
		g_printf (_("(ERROR) Can't save config file!\n"));
	}
	else	{	

/*================================================*
 *			Grupa 'main window position'		  *
 *================================================*/
				
		get_main_win_position ();
		
		g_fprintf (settings_file, "[main window position]\n\n"
							      "remember=%s\n"
						   		  "x=%d\n"
						   		  "y=%d\n",
						   		  convert_gboolean_to_string (settings.remember), 
						   		  settings.x,
						   		  settings.y);

/*================================================*
 *			Grupa 'security copy'				  *
 *================================================*/
		
		if (settings.next_copy_month == 0)	{
			settings.next_copy_month = month + settings.frequency;
			if (settings.next_copy_month > 12)
				settings.next_copy_month = settings.next_copy_month - 12;
		}
		
		g_fprintf (settings_file, "[security copy]\n\n"
							      "auto_copy=%s\n"
						   		  "frequency=%d\n"
						   		  "date=%d\n",
						   		  convert_gboolean_to_string (settings.auto_copy), 
						   		  settings.frequency,
						   		  settings.next_copy_month);
		
		fclose (settings_file);		
		
		set_statusbar_text (_("Settings saved!"));
		g_printf (_("Settings saved!\n"));
	}
}

/*
 *	Funkcja laduje pozycje okna z wartosci podanych w strukturze 'settings'
 */

void		load_main_win_position	(void)	{
	gtk_window_move (GTK_WINDOW (main_window), settings.x, settings.y);
}

/*
 *	Funkcja sprawdza czy ustawiono automatyczne kopie bezpieczenstwa bazy danych
 *	ezeli tak, to pyta uzytkownika czy chce wykonac taaka kopie, tworzy ja i
 *	oblicza termin nastepnej kopie
 */

void		make_security_copy		(void)	{
	if (settings.auto_copy)	{
		if (month == settings.next_copy_month && day == 1)	{
			GtkWidget	*widget;
			gint		result;
			
			widget = create_security_copy_dlg ();

			result = gtk_dialog_run (GTK_DIALOG (widget));
			switch (result)	{
			case GTK_RESPONSE_DELETE_EVENT:
				gtk_widget_destroy (widget);
				break;
			case GTK_RESPONSE_NO:
				gtk_widget_destroy (widget);
				break;
			case GTK_RESPONSE_YES:
				gtk_widget_destroy (widget);
				on_export_data_activate (NULL, NULL);
				break;
			default:
				gtk_widget_destroy (widget);
			}
			
			settings.next_copy_month = month + settings.frequency;
			if (settings.next_copy_month > 12)
				settings.next_copy_month = settings.next_copy_month - 12;
		}
	}
}

/*
 *	Funkcja pobiera aktualna pozycje okna'
 */

void		get_main_win_position	(void)	{
	if (settings.remember)	{
		gtk_window_get_position (GTK_WINDOW (main_window), &settings.x, &settings.y);
	}
}

/*
 *	Funkcja laduje plik konfiguracyjny do okna ustawieni 'settings_dlg'
 */

void		load_settings_dlg			(GtkWidget	*widget)	{
	GtkWidget	*tmp;
	
	tmp = (gpointer) lookup_widget (widget, "set_position_chbox");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), settings.remember);
	
	tmp = (gpointer) lookup_widget (widget, "set_auto_copy_chbox");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), settings.auto_copy);
	on_set_auto_copy_chbox_clicked (GTK_BUTTON (tmp), NULL);
	
	tmp = (gpointer) lookup_widget (widget, "security_month_spin");
	if (!settings.auto_copy)
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (tmp), 1.0);
	else
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (tmp), (gdouble) settings.frequency);
}

/*
 *	Funkcja laduje pozycje okna z wartosci podanych w strukturze 'settings'
 */

void		get_settings_from_dlg	(GtkWidget	*widget)	{
	GtkWidget	*tmp;
	
/*================================================*
 *				Zakladka 'General'				  *
 *================================================*/
	
	tmp = (gpointer) lookup_widget (widget, "set_position_chbox");
	settings.remember = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp));
	
	tmp = (gpointer) lookup_widget (widget, "set_auto_copy_chbox");
	settings.auto_copy = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp));
	
	tmp = (gpointer) lookup_widget (widget, "security_month_spin");
	settings.frequency = (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON (tmp));
}

/*
 *	Funkcja sprawdza stan zmiennej 'gboolean', jezei jest pozytywna zwraca
 *	ciag znakowy 'true', jezeli nie zwraca ciag 'false'
 */

gchar		*convert_gboolean_to_string	(gboolean	variable)	{
	if (variable)
		return "true";
	else
		return "false";
}
